@@ -255,18 +255,25 @@ public:
     void clockSilent(int cycles);
 
     /**
-     * Get chip's 6581 filter.
+     * Set filter curve parameter for 6581 model.
      *
-     * @return filter
+     * @see Filter6581::setFilterCurve(double)
      */
-    Filter6581* getFilter6581() const { return filter6581; }
+    void setFilter6581Curve(double filterCurve);
 
     /**
-     * Get chip's 8580 filter.
+     * Set filter curve parameter for 8580 model.
      *
-     * @return filter
+     * @see Filter8580::setFilterCurve(double)
      */
-    Filter8580* getFilter8580() const { return filter8580; }
+    void setFilter8580Curve(double filterCurve);
+
+    /**
+     * Enable filter emulation.
+     *
+     * @param enable false to turn off filter emulation
+     */
+    void enableFilter(bool enable);
 };
 
 } // namespace reSIDfp
@@ -283,7 +290,7 @@ public:
 namespace reSIDfp
 {
 
-RESID_INLINE  
+RESID_INLINE
 void SID::ageBusValue(int n)
 {
     if (likely(busValueTtl != 0))
@@ -301,13 +308,11 @@ void SID::ageBusValue(int n)
 RESID_INLINE
 int SID::output() const
 {
-    return externalFilter->clock(
-               filter->clock(
-                   voice[0]->output(voice[2]->wave()),
-                   voice[1]->output(voice[0]->wave()),
-                   voice[2]->output(voice[1]->wave())
-               )
-           );
+    const int v1 = voice[0]->output(voice[2]->wave());
+    const int v2 = voice[1]->output(voice[0]->wave());
+    const int v3 = voice[2]->output(voice[1]->wave());
+
+    return externalFilter->clock(filter->clock(v1, v2, v3));
 }
 
 
