@@ -64,9 +64,9 @@ matrix_t* WaveformCalculator::buildTable(ChipModel model)
 
     matrix_t wftable(8, 4096);
 
-    for (int accumulator = 0; accumulator < 1 << 24; accumulator += 1 << 12)
+    for (unsigned int accumulator = 0; accumulator < 1 << 24; accumulator += 1 << 12)
     {
-        const int idx = (accumulator >> 12);
+        const int unsigned idx = (accumulator >> 12);
         wftable[0][idx] = 0xfff;
         wftable[1][idx] = (short)((accumulator & 0x800000) == 0 ? idx << 1 : (idx ^ 0xfff) << 1);
         wftable[2][idx] = (short) idx;
