@@ -1,7 +1,8 @@
 /*
  * This file is part of libsidplayfp, a SID player engine.
  *
- * Copyright 2011-2013 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2011-2014 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2009-2014 VICE Project
  * Copyright 2007-2010 Antti Lankila
  * Copyright 2001 Simon White
  *
@@ -23,11 +24,13 @@
 #ifndef MOS656X_H
 #define MOS656X_H
 
-#include <stdint.h> 
+#include <stdint.h>
 
-#include "sidplayfp/event.h" 
-#include "sidplayfp/component.h"
-#include "sidplayfp/EventScheduler.h"
+#include "lightpen.h"
+#include "sprites.h"
+#include "sidplayfp/event.h"
+#include "c64/component.h"
+#include "EventScheduler.h"
 
 
 class MOS656X: public component, private Event
@@ -35,10 +38,10 @@ class MOS656X: public component, private Event
 public:
     typedef enum
     {
-        MOS6567R56A = 0  /* OLD NTSC CHIP */
-        ,MOS6567R8       /* NTSC-M */
-        ,MOS6569         /* PAL-B */
-        ,MOS6572         /* PAL-N */
+        MOS6567R56A = 0  ///< OLD NTSC CHIP
+        ,MOS6567R8       ///< NTSC-M
+        ,MOS6569         ///< PAL-B
+        ,MOS6572         ///< PAL-N
     } model_t;
 
 private:
@@ -54,75 +57,69 @@ private:
 
     static const model_data_t modelData[];
 
-private:
-    /** raster IRQ flag */
+    /// raster IRQ flag
     static const int IRQ_RASTER = 1 << 0;
 
-    /** Light-Pen IRQ flag */
+    /// Light-Pen IRQ flag
     static const int IRQ_LIGHTPEN = 1 << 3;
 
-protected:
-    /** First line when we check for bad lines */
+    /// First line when we check for bad lines
     static const unsigned int FIRST_DMA_LINE = 0x30;
 
-    /** Last line when we check for bad lines */
+    /// Last line when we check for bad lines
     static const unsigned int LAST_DMA_LINE = 0xf7;
 
-protected:
+private:
     event_clock_t (MOS656X::*clock)();
 
     event_clock_t rasterClk;
 
-    /** CPU's event context. */
+    /// CPU's event context.
     EventContext &event_context;
 
-    /** Number of cycles per line. */
+    /// Number of cycles per line.
     unsigned int cyclesPerLine;
 
+    /// Number of raster lines.
     unsigned int maxRasters;
 
-    /** Current visible line */
+    /// Current visible line
     unsigned int lineCycle;
 
-    /** current raster line */
+    /// current raster line
     unsigned int rasterY;
 
-    /** vertical scrolling value */
+    /// vertical scrolling value
     unsigned int yscroll;
 
-    /** are bad lines enabled for this frame? */
+    /// are bad lines enabled for this frame?
     bool areBadLinesEnabled;
 
-    /** is the current line a bad line */
+    /// is the current line a bad line
     bool isBadLine;
 
-    /** Is rasterYIRQ condition true? */
+    /// Is rasterYIRQ condition true?
     bool rasterYIRQCondition;
 
-    /** Set when new frame starts. */
+    /// Set when new frame starts.
     bool vblanking;
 
-    /** Has light pen IRQ been triggered in this frame already? */
-    bool lp_triggered;
+    /// Is CIA asserting lightpen?
+    bool lpAsserted;
 
-    /** internal IRQ flags */
+    /// internal IRQ flags
     uint8_t irqFlags;
 
-    /** masks for the IRQ flags */
+    /// masks for the IRQ flags
     uint8_t irqMask;
 
-    /** Light pen coordinates */
-    uint8_t lpx, lpy;
+    /// Light pen coordinates
+    Lightpen lp;
 
-    /** the 8 sprites data*/
-    //@{
-    uint8_t &sprite_enable, &sprite_y_expansion;
-    uint8_t sprite_dma;
-    uint8_t sprite_mc_base[8];
-    uint8_t sprite_mc[8];
-    //@}
+    /// the 8 sprites data
+    Sprites sprites;
 
-    /** memory for chip registers */
+    /// memory for chip registers
     uint8_t regs[0x40];
 
 private:
@@ -130,17 +127,23 @@ private:
     event_clock_t clockNTSC();
     event_clock_t clockOldNTSC();
 
-    /** Signal CPU interrupt if requested by VIC. */
+    /**
+     * Signal CPU interrupt if requested by VIC.
+     */
     void handleIrqState();
 
     EventCallback<MOS656X> badLineStateChangeEvent;
 
-    /** AEC state was updated. */
+    /**
+     * AEC state was updated.
+     */
     void badLineStateChange() { setBA(!isBadLine); }
 
     EventCallback<MOS656X> rasterYIRQEdgeDetectorEvent;
 
-    /** RasterY IRQ edge detector. */
+    /**
+     * RasterY IRQ edge detector.
+     */
     void rasterYIRQEdgeDetector()
     {
         const bool oldRasterYIRQCondition = rasterYIRQCondition;
@@ -148,11 +151,11 @@ private:
         if (!oldRasterYIRQCondition && rasterYIRQCondition)
             activateIRQFlag(IRQ_RASTER);
     }
-    
+
     /**
-    * Set an IRQ flag and trigger an IRQ if the corresponding IRQ mask is set.
-    * The IRQ only gets activated, i.e. flag 0x80 gets set, if it was not active before.
-    */
+     * Set an IRQ flag and trigger an IRQ if the corresponding IRQ mask is set.
+     * The IRQ only gets activated, i.e. flag 0x80 gets set, if it was not active before.
+     */
     void activateIRQFlag(int flag)
     {
         irqFlags |= flag;
@@ -160,28 +163,29 @@ private:
     }
 
     /**
-    * Read the value of the raster line IRQ
-    * 
-    * @return raster line when to trigger an IRQ
-    */
+     * Read the value of the raster line IRQ
+     *
+     * @return raster line when to trigger an IRQ
+     */
     unsigned int readRasterLineIRQ() const
     {
         return (regs[0x12] & 0xff) + ((regs[0x11] & 0x80) << 1);
     }
 
     /**
-    * Read the DEN flag which tells whether the display is enabled
-    *
-    * @return true if DEN is set, otherwise false
-    */
+     * Read the DEN flag which tells whether the display is enabled
+     *
+     * @return true if DEN is set, otherwise false
+     */
     bool readDEN() const { return (regs[0x11] & 0x10) != 0; }
 
-    bool evaluateIsBadLine() const
+    /**
+     * Get previous value of Y raster
+     */
+    inline unsigned int oldRasterY()
     {
-        return areBadLinesEnabled
-            && rasterY >= FIRST_DMA_LINE
-            && rasterY <= LAST_DMA_LINE
-            && (rasterY & 7) == yscroll;
+        const int prevRasterY = rasterY - 1;
+        return prevRasterY >= 0 ? prevRasterY : cyclesPerLine - 1;
     }
 
     inline void sync()
@@ -190,23 +194,38 @@ private:
         event();
     }
 
+    /**
+     * Check for vertical blanking.
+     */
     inline void checkVblank()
     {
         // IRQ occurred (xraster != 0)
         if (rasterY == (maxRasters - 1))
+        {
             vblanking = true;
-        else
+        }
+
+        // Check DEN bit on first cycle of the line following the first DMA line
+        if (rasterY == FIRST_DMA_LINE
+            && !areBadLinesEnabled
+            && readDEN())
+        {
+            areBadLinesEnabled = true;
+        }
+
+        // Disallow bad lines after the last possible one has passed
+        if (rasterY == LAST_DMA_LINE)
+        {
+            areBadLinesEnabled = false;
+        }
+
+        isBadLine = false;
+
+        if (!vblanking)
         {
             rasterY++;
             rasterYIRQEdgeDetector();
         }
-
-        // In line $30, the DEN bit controls if Bad Lines can occur
-        if (rasterY == FIRST_DMA_LINE)
-            areBadLinesEnabled = readDEN();
-
-        // Test for bad line condition
-        isBadLine = evaluateIsBadLine();
     }
 
     inline void vblank()
@@ -214,92 +233,40 @@ private:
         // Vertical blank (line 0)
         if (vblanking)
         {
-            vblanking = lp_triggered = false;
+            vblanking = false;
             rasterY = 0;
             rasterYIRQEdgeDetector();
-        }
-    }
-
-    inline void updateMc()
-    {
-        // Update mc values in one pass
-        // after the dma has been processed
-        uint8_t mask = 1;
-        for (unsigned int i=0; i<8; i++, mask<<=1)
-        {
-            if (sprite_dma & mask)
-                sprite_mc[i] = (sprite_mc[i] + 3) & 0x3f;
-        }
-    }
-
-    inline void updateMcBase()
-    {
-        uint8_t mask = 1;
-        for (unsigned int i=0; i<8; i++, mask<<=1)
-        {
-            if (sprite_y_expansion & mask)
-                sprite_mc_base[i] = sprite_mc[i];
-            if (sprite_mc_base[i] == 0x3f)
-                sprite_dma &= ~mask;
-        }
-    }
-
-    /// Calculate sprite DMA and sprite expansion
-    inline void checkSpriteDmaExp()
-    {
-        const uint8_t y = rasterY & 0xff;
-        uint8_t mask = 1;
-        for (unsigned int i=0; i<8; i++, mask<<=1)
-        {
-            if ((sprite_enable & mask) && (y == regs[(i << 1) + 1]))
+            lp.untrigger();
+            if (lpAsserted && lp.retrigger(lineCycle, rasterY))
             {
-                sprite_dma |= mask;
-                sprite_mc_base[i] = 0;
-                sprite_y_expansion |= mask;
+                activateIRQFlag(IRQ_LIGHTPEN);
             }
         }
     }
 
-    /// Calculate sprite DMA
-    inline void checkSpriteDma()
-    {
-        const uint8_t y = rasterY & 0xff;
-        uint8_t mask = 1;
-        for (unsigned int i=0; i<8; i++, mask<<=1)
-        {
-            if ((sprite_enable & mask) && (y == regs[(i << 1) + 1]) && !(sprite_dma & mask))
-            {
-                sprite_dma |= mask;
-                sprite_mc_base[i] = 0;
-            }
-        }
-    }
-
-    inline void checkSpriteDisplay()
-    {
-        for (unsigned int i=0; i<8; i++)
-        {
-            sprite_mc[i] = sprite_mc_base[i];
-        }
-    }
-
-    /// Start DMA for sprite n
+    /**
+     * Start DMA for sprite n.
+     */
     template<int n>
     inline void startDma()
     {
-        if (sprite_dma & (0x01 << n))
+        if (sprites.isDma(0x01 << n))
             setBA(false);
     }
 
-    /// End DMA for sprite n
+    /**
+     * End DMA for sprite n.
+     */
     template<int n>
     inline void endDma()
     {
-        if (!(sprite_dma & (0x06 << n)))
+        if (!sprites.isDma(0x06 << n))
             setBA(true);
     }
 
-    /// Start bad line
+    /**
+     * Start bad line.
+     */
     inline void startBadline()
     {
         if (isBadLine)
@@ -315,45 +282,58 @@ protected:
     virtual void setBA     (bool state) = 0;
 
     /**
-    * Read VIC register.
-    *
-    * @param addr
-    *            Register to read.
-    */
+     * Read VIC register.
+     *
+     * @param addr
+     *            Register to read.
+     */
     uint8_t read(uint_least8_t addr);
 
     /**
-    * Write to VIC register.
-    *
-    * @param addr
-    *            Register to write to.
-    * @param data
-    *            Data byte to write.
-    */
+     * Write to VIC register.
+     *
+     * @param addr
+     *            Register to write to.
+     * @param data
+     *            Data byte to write.
+     */
     void write(uint_least8_t addr, uint8_t data);
 
 public:
     void event();
 
     void chip(model_t model);
-    void lightpen();
+
+    /**
+     * Trigger the lightpen. Sets the lightpen usage flag.
+     */
+    void triggerLightpen();
+
+    /**
+     * Clears the lightpen usage flag.
+     */
+    void clearLightpen();
 
     // Component Standard Calls
     void reset();
 
-    const char *credits() const { return credit; }
+    static const char *credits() { return credit; }
 };
 
 // Template specializations
 
-/// Start DMA for sprite 0
+/**
+ * Start DMA for sprite 0.
+ */
 template<>
 inline void MOS656X::startDma<0>()
 {
-    setBA(!(sprite_dma & 0x01));
+    setBA(!sprites.isDma(0x01));
 }
 
-/// End DMA for sprite 7
+/**
+ * End DMA for sprite 7.
+ */
 template<>
 inline void MOS656X::endDma<7>()
 {
