@@ -37,32 +37,95 @@
 const char ERR_PSIDDRV_NO_SPACE[]  = "ERROR: No space to install psid driver in C64 ram";
 const char ERR_PSIDDRV_RELOC[]     = "ERROR: Failed whilst relocating psid driver";
 
-uint8_t psiddrv::psid_driver[] = {
+uint8_t psid_driver[] = {
 #  include "psiddrv.bin"
 };
 
+const uint8_t POWERON[] =
+{
+#  include "poweron.bin"
+};
+
+
+/**
+ * Copy power on pattern in memory.
+ */
+void copyPoweronPattern(sidmemory *mem)
+{
+    // Copy in power on settings. These were created by running
+    // the kernel reset routine and storing the useful values
+    // from $0000-$03ff. Format is:
+    // - offset byte (bit 7 indicates presence rle byte)
+    // - rle count byte (bit 7 indicates compression used)
+    // - data (single byte) or quantity represented by uncompressed count
+    // all counts and offsets are 1 less than they should be
+    uint_least16_t addr = 0;
+    for (unsigned int i = 0; i < sizeof(POWERON);)
+    {
+        uint8_t off   = POWERON[i++];
+        uint8_t count = 0;
+        bool compressed = false;
+
+        // Determine data count/compression
+        if (off & 0x80)
+        {
+            // fixup offset
+            off  &= 0x7f;
+            count = POWERON[i++];
+            if (count & 0x80)
+            {
+                // fixup count
+                count &= 0x7f;
+                compressed = true;
+            }
+        }
+
+        // Fix count off by ones (see format details)
+        count++;
+        addr += off;
+
+        // Extract compressed data
+        if (compressed)
+        {
+            const uint8_t data = POWERON[i++];
+            while (count-- > 0)
+            {
+                mem->writeMemByte(addr++, data);
+            }
+        }
+        // Extract uncompressed data
+        else
+        {
+            while (count-- > 0)
+            {
+                mem->writeMemByte(addr++, POWERON[i++]);
+            }
+        }
+    }
+}
 
 uint8_t psiddrv::iomap(uint_least16_t addr) const
 {
     // Force Real C64 Compatibility
-    switch (m_tuneInfo->compatibility())
+    if (m_tuneInfo->compatibility() == SidTuneInfo::COMPATIBILITY_R64
+        || m_tuneInfo->compatibility() == SidTuneInfo::COMPATIBILITY_BASIC
+        || addr == 0)
     {
-    case SidTuneInfo::COMPATIBILITY_R64:
-    case SidTuneInfo::COMPATIBILITY_BASIC:
-        return 0;     // Special case, converted to 0x37 later
+        // Special case, converted to 0x37 by the psid driver
+        return 0;
     }
 
-    if (addr == 0)
-        return 0;     // Special case, converted to 0x37 later
+    if (addr < 0xa000)
+        return 0x37;  // Basic-ROM, Kernal-ROM, I/O
+    if (addr < 0xd000)
+        return 0x36;  // Kernal-ROM, I/O
     if (addr >= 0xe000)
         return 0x35;  // I/O only
-    if (addr >= 0xd000)
-        return 0x34;  // RAM only
 
-    return 0x36;  // Kernal-ROM, I/O
+    return 0x34;  // RAM only
 }
 
-bool psiddrv::drvReloc(sidmemory *mem)
+bool psiddrv::drvReloc()
 {
     const int startlp = m_tuneInfo->loadAddr() >> 8;
     const int endlp   = (m_tuneInfo->loadAddr() + (m_tuneInfo->c64dataLen() - 1)) >> 8;
@@ -70,11 +133,9 @@ bool psiddrv::drvReloc(sidmemory *mem)
     uint_least8_t relocStartPage = m_tuneInfo->relocStartPage();
     uint_least8_t relocPages = m_tuneInfo->relocPages();
 
-    // Will get done later if can't now
-    mem->writeMemByte(0x02a6, (m_tuneInfo->clockSpeed() == SidTuneInfo::CLOCK_PAL) ? 1 : 0);
-
     if (m_tuneInfo->compatibility() == SidTuneInfo::COMPATIBILITY_BASIC)
-    {   // The psiddrv is only used for initialisation and to
+    {
+        // The psiddrv is only used for initialisation and to
         // autorun basic tunes as running the kernel falls
         // into a manual load/run mode
         relocStartPage = 0x04;
@@ -88,8 +149,8 @@ bool psiddrv::drvReloc(sidmemory *mem)
     else if (relocStartPage == 0)
     {
         relocPages = 0;
-        /* find area where to dump the driver in.
-        * It's only 1 block long, so any free block we can find will do. */
+        // find area where to dump the driver in.
+        // It's only 1 block long, so any free block we can find will do.
         for (int i = 4; i < 0xd0; i ++)
         {
             if (i >= startlp && i <= endlp)
@@ -127,23 +188,38 @@ bool psiddrv::drvReloc(sidmemory *mem)
 
     // Adjust size to not included initialisation data.
     reloc_size -= 10;
+
     m_driverAddr   = relocAddr;
     m_driverLength = (uint_least16_t)reloc_size;
     // Round length to end of page
     m_driverLength += 0xff;
     m_driverLength &= 0xff00;
 
+    return true;
+}
+
+void psiddrv::install(sidmemory *mem, uint8_t video) const
+{
+    if (m_tuneInfo->compatibility() >= SidTuneInfo::COMPATIBILITY_R64)
+    {
+        copyPoweronPattern(mem);
+    }
+
+    mem->writeMemByte(0x02a6, video);
+
     mem->installResetHook(endian_little16(reloc_driver));
 
     // If not a basic tune then the psiddrv must install
     // interrupt hooks and trap programs trying to restart basic
     if (m_tuneInfo->compatibility() == SidTuneInfo::COMPATIBILITY_BASIC)
-    {   // Install hook to set subtune number for basic
-        mem->setBasicSubtune((uint8_t) (m_tuneInfo->currentSong() - 1));
+    {
+        // Install hook to set subtune number for basic
+        mem->setBasicSubtune((uint8_t)(m_tuneInfo->currentSong() - 1));
         mem->installBasicTrap(0xbf53);
     }
     else
-    {   // Only install irq handle for RSID tunes
+    {
+        // Only install irq handle for RSID tunes
         mem->fillRam(0x0314, &reloc_driver[2], m_tuneInfo->compatibility() == SidTuneInfo::COMPATIBILITY_R64 ? 2 : 6);
 
         // Experimental restart basic trap
@@ -152,35 +228,40 @@ bool psiddrv::drvReloc(sidmemory *mem)
         mem->writeMemWord(0x0328, addr);
     }
 
-    return true;
-}
-
-void psiddrv::install(sidmemory *mem) const
-{
     int pos = m_driverAddr;
 
     // Install driver to ram
     mem->fillRam(pos, &reloc_driver[10], reloc_size);
 
-    // Tell C64 about song
+    // Set song number
     mem->writeMemByte(pos, (uint8_t) (m_tuneInfo->currentSong() - 1));
     pos++;
+
+    // Set speed flag
     mem->writeMemByte(pos, m_tuneInfo->songSpeed() == SidTuneInfo::SPEED_VBI ? 0 : 1);
     pos++;
+
+    // Set init address
     mem->writeMemWord(pos, m_tuneInfo->compatibility() == SidTuneInfo::COMPATIBILITY_BASIC ?
-                     0xbf55 /*Was 0xa7ae, see above*/ : m_tuneInfo->initAddr());
+                     0xbf55 : m_tuneInfo->initAddr());
     pos += 2;
+
+    // Set play address
     mem->writeMemWord(pos, m_tuneInfo->playAddr());
     pos += 2;
     mem->writeMemWord(pos, m_powerOnDelay);
     pos += 2;
 
+    // Set init address io bank value
     mem->writeMemByte(pos, iomap(m_tuneInfo->initAddr()));
     pos++;
+
+    // Set play address io bank value
     mem->writeMemByte(pos, iomap(m_tuneInfo->playAddr()));
     pos++;
-    const uint8_t flag = mem->readMemByte(0x02a6); // PAL/NTSC flag
-    mem->writeMemByte(pos, flag);
+
+    // Set PAL/NTSC flag
+    mem->writeMemByte(pos, video);
     pos++;
 
     // Add the required tune speed
@@ -193,11 +274,11 @@ void psiddrv::install(sidmemory *mem) const
         mem->writeMemByte(pos, 0);
         break;
     default: // UNKNOWN or ANY
-        mem->writeMemByte(pos, flag);
+        mem->writeMemByte(pos, video);
         break;
     }
     pos++;
 
-    // Default processor register flags on calling init
+    // Set default processor register flags on calling init
     mem->writeMemByte(pos, m_tuneInfo->compatibility() >= SidTuneInfo::COMPATIBILITY_R64 ? 0 : 1 << MOS6510::SR_INTERRUPT);
 }
